#!BPY

# $Id$
"""
Name :'MarbleCLAY(mcl)'
Blender: 234
Group: 'Export'
Submenu: 'With VertexGroup' on
Submenu: 'Without VertexGroup' off
Tip: 'Export to MarbleCLAY File Format'
"""
__author__="Nao"
__url__="None"
__version__="2005/04/15/1"
__bpydoc__="""\
This is an another Blender -> MarbleCLAY exporter.
"""

#***********************************************************
# Blender Mesh to MarbleCLAY beta 5.4
# Original MCL_Exporter (2004/10/15) by Nao
# fix Material (2004/12/29) by Nao
# add VertexGroup (2005/03/28) by Nao
#***********************************************************

#***********************************************************
# Import Module
#***********************************************************
import Blender
from Blender import Types, Object, NMesh, Texture, Material, Draw
from math import *
from string import *

#***********************************************************
# Main
#***********************************************************
def export(filename):
	print filename+" Exporting"
	file = open(filename,"w")
	startScene(file)
	imgList = []
	texmapOut(file,imgList)
	materialsOut(file,imgList)
	meshOut(file)
	file.close()
	print "finish",'\a\r'

#***********************************************************
# Header write
#***********************************************************
def startScene(file):
	file.write("Header {\n  CreatedBy MCL_Exporter\n}\n")
	file.write(" Document {\n")
	file.write("   Vx.0 45\n   Hy.0 20\n   Rz.0 200\n")
	file.write("   ViewCenter.x.0 0\n   ViewCenter.y.0 0\n   ViewCenter.z.0 0\n")
	file.write("   ViewAngle0 30\n")
	file.write("   Light.x.0 0\n   Light.y.0 0\n   Light.z.0 1\n")
	file.write(" }\n")

#***********************************************************
# Texture filename write
#***********************************************************
def texmapOut(file,imgList):
	imgHash = {}
	id = 0
	for mat in Blender.Material.Get():
		mtextures = mat.getTextures()
		for mtex in mtextures:
			if mtex is not None and mtex.tex.type == Texture.Types.IMAGE:
				ImageProperty = mtex.tex.image.filename
				try:
					dummy = imgHash[ImageProperty]
				except:
					imgHash[ImageProperty] = id
					imgList.append([mtex.tex.image.size[0],mtex.tex.image.size[1],mtex.tex.image.filename])
					id += 1
	for IDX in range(len(imgList)):
		print IDX,imgList[IDX][0],imgList[IDX][1],imgList[IDX][2]
		file.write('Bitmap {\n')
		temp = '  '+str(IDX)+' '+str(imgList[IDX][0])+' '+str(imgList[IDX][1])+' '+imgList[IDX][2]+'\n}\n'
		file.write(temp)

#***********************************************************
# Material wite
#***********************************************************
def materialsOut(file,imgList):
	print "Material",
	if Blender.Material.Get():
		file.write("Materials {\n")
	for mat in Blender.Material.Get():
		mtextures = mat.getTextures()
		file.write("  Material {\n")
		file.write("    name \"%s\"\n" % mat.name)
		file.write("    Material {\n")
		file.write("      name Diffuse\n")
		file.write("      col3 %s %s %s\n" % (round(mat.R*255), round(mat.G*255), round(mat.B*255)))
		file.write("    }\n")
		file.write("    Material {\n")
		file.write("      name Specular\n")
		file.write("      val1 %s\n" % (mat.spec))
		file.write("    }\n")
		file.write("    Material {\n")
		file.write("      name Shininess\n")
		file.write("      val1 %s\n" % (mat.hard / 10))
		file.write("    }\n")
		file.write("    Material {\n")
		file.write("      name Alpha\n")
		file.write("      val1 %s\n" % (mat.alpha))
		file.write("    }\n")
		file.write("    Material {\n")
		file.write("      name Ambient\n")
		file.write("      col3 %s %s %s\n" % (mat.amb, mat.amb, mat.amb))
		file.write("    }\n")
		file.write("    Material {\n")
		file.write("      name Glow\n")
		file.write("      col3 %s %s %s\n" % (round(mat.emit*255), round(mat.emit*255), round(mat.emit*255)))
		file.write("    }\n")
		file.write("    Material {\n")
		file.write("      name Refrection\n")
		file.write("      col3 %s %s %s\n" % (round(mat.rayMirr*255), round(mat.rayMirr*255), round(mat.rayMirr*255)))
		file.write("    }\n")
		file.write("    Material {\n")
		file.write("      name Refraction\n")
		file.write("      val1 %s\n" % (mat.IOR))
		file.write("    }\n")
		mapFLG = 0
		for mtex in mtextures:
			if mtex is not None and mtex.tex.type == Texture.Types.IMAGE and mtex.mapto == Texture.MapTo.COL:
				for IDX in range(len(imgList)):
					if mtex.tex.image.filename == imgList[IDX][2]:
						file.write("    Material {\n")
						file.write("      map "+str(IDX)+"\n")
						file.write("    }\n")
						mapFLG = 1
		if mapFLG == 0:
			file.write("    Material {\n")
			file.write("      map -1\n")
			file.write("    }\n")
		file.write("  }\n")
	file.write("}\n")
	print "OK"

#***********************************************************
# Mesh Raw write
#***********************************************************
def meshOut(file):
	LayMask = GetActiveLayer()
	for ob in Blender.Object.Get():
		if (LayMask & ob.Layer) == ob.Layer:
			if ob.getType() == 'Mesh':
				namae = ob.getName()
				print namae,
				temp = ""
				Mesh = Blender.NMesh.GetRawFromObject(namae)
				file.write("Object {\n   Structure {\n")
				file.write("     Name \"%s\"\n" % Mesh.name)
				file.write("     Stat 1 0\n")
				TMesh = ob.getData()
				if __script__['arg'] == 'on' and len(TMesh.getVertGroupNames()) > 0:
					#print TMesh.getVertGroupNames()
					GName = TMesh.getVertGroupNames()
					for GNum in range(len(GName)):
						file.write("     Mesh \""+GName[GNum]+"\" "+str(GNum)+" 1 0\n")
					file.write("   }\n")
					file.write("  Vertexs {\n")
					vIndex = 0
					for vertex in Mesh.verts:
						vert = MatrixTransform(vertex,ob.matrix)
						for GNum in range(len(GName)):
							if TMesh.getVertsFromGroup(GName[GNum],0,[vIndex]):
								temp = temp + "     %s %f,%f,%f 0 -1 0.3333\n" % (GNum,(vert[1]*-1),vert[0],vert[2])
								break
						vIndex += 1
				else:
					file.write("     Mesh \"Sub-Mesh\" 0 1 0\n")
					file.write("   }\n")
					file.write("  Vertexs {\n")
					for vertex in Mesh.verts:
						vert = MatrixTransform(vertex,ob.matrix)
						temp = temp + "    0 %f,%f,%f 0 -1 0.3333\n" % ((vert[1]*-1),vert[0],vert[2])
				file.write(temp)
				file.write("  }\n")
				if Mesh.hasFaceUV():
					file.write("  UVs {\n")
					temp = ""
					for vIndex in range(len(Mesh.verts)):
						for fac in Mesh.faces:
							for nv in range(len(fac.v)):
								if fac.v[nv].index == vIndex:
									temp = temp + "    0 0,%f,%f 0 -1 %s\n" % (fac.uv[nv][0],fac.uv[nv][1]*-1,vIndex)
									break
					file.write(temp)
					file.write("  }\n")
				file.write("  Polygons {\n")
				#print Mesh.materials
				for fac in Mesh.faces:
					temp = ""
					fmat = 0
					#print Blender.Material.Get(),fac.materialIndex
					if len(Mesh.materials) < fac.materialIndex:
						print fac.materialIndex
					else:
						for mat in Blender.Material.Get():
							if mat.name == Mesh.materials[fac.materialIndex].name: break
							else: fmat +=1
					#fmat = len(Blender.Material.Get()) - fac.materialIndex
					nv = len(fac.v)
					if __script__['arg'] == 'on' and len(TMesh.getVertGroupNames()) > 0:
						GName = TMesh.getVertGroupNames()
						for GNum in range(len(GName)):
							if TMesh.getVertsFromGroup(GName[GNum],0,[fac.v[0].index]):
								if nv == 4:
									temp = temp + "    %s %s,%s,%s,%s 0 0 -1 -1,-1,-1,-1 %s\n" % (GNum,fac.v[3].index,fac.v[2].index,fac.v[1].index,fac.v[0].index,fmat)
								else:
									temp = temp + "    %s %s,%s,%s 0 0 -1 -1,-1,-1,-1 %s\n" % (GNum,fac.v[2].index,fac.v[1].index,fac.v[0].index,fmat)
								break
					else:
						if nv == 4:temp = temp + "    0 %s,%s,%s,%s 0 0 -1 -1,-1,-1,-1 %s\n" % (fac.v[3].index,fac.v[2].index,fac.v[1].index,fac.v[0].index,fmat)
						else:temp = temp + "    0 %s,%s,%s 0 0 -1 -1,-1,-1 %s\n" % (fac.v[2].index,fac.v[1].index,fac.v[0].index,fmat)
					file.write(temp)
				file.write("  }\n")
				file.write("}\n")
				print " OK"

#***********************************************************
# Matrix Option
#***********************************************************
def MatrixTransform(vert,matrix):
	x, y, z = vert
	retx = x*matrix[0][0] + y*matrix[1][0] + z*matrix[2][0] + matrix[3][0]
	rety = x*matrix[0][1] + y*matrix[1][1] + z*matrix[2][1] + matrix[3][1]
	retz = x*matrix[0][2] + y*matrix[1][2] + z*matrix[2][2] + matrix[3][2]
	vert = [retx,rety,retz]
	return vert

#***********************************************************
# Layer Option
#***********************************************************
def GetActiveLayer():
	maskColection = [0,1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768,65536,131072,262144,524288,1048573]
	Layermask = 0
	layers = Blender.Window.ViewLayer()
	for i in range(len(layers)):
		Layermask = Layermask + maskColection[layers[i]]
	return Layermask

#***********************************************************
# File Selection
#***********************************************************
def fs_callback(filename):
	if filename.find('.mcl', -4) <= 0: filename += '.mcl'
	export(filename)
Blender.Window.FileSelector(fs_callback,"Save mcl")

