#!BPY
"""
Name: 'MarbleCLAY(mcl)'
Blender: 234
Group: 'Import'
Tooltip: 'Import MarbleCLAY format'
"""

#***********************************************************
# MarbleCLAY Mesh Importer (With out Bone)
# Original MCL_Importer.py (2004/11/06) by Nao
# add Vertex Group (2004/11/10) by Nao
# add Texture Map (2004/11/12) by Nao
# fix Material (2004/12/29) by Nao
#***********************************************************

#***********************************************************
# Import Module
#***********************************************************
import Blender
from Blender import Object, NMesh, Material

#***********************************************************
# Main
#***********************************************************
def Import(filename):
	print ''
	print filename+" Importing"
	DefMat = Material.New("DefMat")
	DefMat.rgbCol = 0.5, 0.5, 1
	MatCnt = len(Blender.Material.Get())
	MapList = []
	file = open(filename,"rb")
	Line = file.readline()
	while "" != Line:
		if len(Line) == 0: pass
		elif Line.find("Bitmap") != -1:
			BitmapGet(file,MapList)
		elif Line.find("Materials") != -1:
			MaterialGet(file,MapList)
		elif Line.find("Object") != -1:
			ObjectGet(file,MatCnt)
			Blender.Redraw()
		Line = file.readline()
	file.close()
	Blender.Redraw()
	print "finish"#,'\a\r'

#***********************************************************
# BitmapGet 
#***********************************************************
def BitmapGet(file,MapList):
	ENFLG = 1
	Line = file.readline()
	while ENFLG == 1:
		if len(Line) == 0: pass
		elif (Line.find("bin")) != -1:
			ENFLG = 0
		elif ENFLG == 1:
			FName = ''
			if Line.find("\"") != -1:
				temp = Line.split("\"")
				FName = temp[1]
			else:
				temp = Line.split()
				cnt = 3
				for i in range(len(temp)-3):
					FName = FName + temp[cnt]
					cnt += 1
			MapList.append(FName)
			ENFLG = 0

#***********************************************************
# MaterialGet
#***********************************************************
def MaterialGet(file,MapList):
	print "Materials"
	Line = file.readline()
	while Line[0] != "}":
		if len(Line) == 0: pass
		elif (Line.find("name") != -1) & (Line.find('"') != -1):
			MatDiff,MatSpc,MatShn,MatBum,MatTrn,MatAlp,MatAmb,MatGlw,MatRefC,MatRef = 0,0,0,0,0,0,0,0,0,0
			temp = Line.split()
			matName = temp[1].strip("\"")
			print "Material "+ matName
			mat = Material.New(matName)
		elif Line.find("Diffuse") != -1:
			if Line.find("Map") == -1:
				MatDiff = 1
			else:
				MatDiff = 2
		elif Line.find("Specular") != -1:
			MatSpc = 1
		elif Line.find("Shininess") != -1:
			MatShn = 1
		elif Line.find("Bump") != -1:
			if Line.find("Map") == -1:
				MatBum = 1
			else:
				MatBum = 2
		elif Line.find("Transparency") != -1:
			if Line.find("Map") == -1:
				MatTrn = 1
			else:
				MatTrn = 2
		elif Line.find("Alpha") != -1:
			if Line.find("Map") == -1:
				MatAlp = 1
			else:
				MatAlp = 2
		elif Line.find("Ambient") != -1:
			if Line.find("Map") == -1:
				MatAmb = 1
			else:
				MatAmb = 2
		elif Line.find("Glow") != -1:
			if Line.find("Map") == -1:
				MatGlw = 1
			else:
				MatGlw = 2
		elif Line.find("Refrection") != -1:
			if Line.find("Map") == -1:
				MatRefC = 1
			else:
				MatRefC = 2
		elif Line.find("Refraction") != -1:
			if Line.find("Map") == -1:
				MatRef = 1
			else:
				MatRef = 2
		elif Line.find("col3") != -1:
			temp = Line.split()
			if float(temp[1]) > 0: R = float(temp[1]) / 255
			else: R = 0
			if float(temp[2]) > 0: G = float(temp[2]) / 255
			else: G = 0
			if float(temp[3]) > 0: B = float(temp[3]) / 255
			else: B = 0
			if MatDiff == 1:
				mat.rgbCol = R,G,B
				MatDiff = 0
			elif MatGlw == 1:
				GVal = (R+G+B) / 3
				mat.emit = float(GVal)
				mat.add = float(GVal)
				MatGlw = 0
			elif MatTrn == 1:
				GVal = (R+G+B) / 3
				mat.specTransp = GVal
				MatTrn = 0
			elif MatAmb == 1:
				GVal = (R+G+B) / 3
				mat.amb = float(GVal)
				MatAmb = 0
			elif MatRefC == 1:
				GVal = (R+G+B) / 3
				mat.rayMirr = float(GVal)
				#mat.mirCol = R,G,B
				MatRefC = 0
		elif Line.find("val1") != -1:
			temp = Line.split()
			GVal = temp[1]
			if MatSpc == 1:
				mat.spec = float(GVal)
				MatSpc = 0
			elif MatShn == 1:
				mat.hard = int(float(GVal) * 10)
				MatShn = 0
			elif MatBum == 1:
				MatBum = 0
			elif MatAlp == 1:
				mat.alpha = float(GVal)
				MatAlp = 0
			elif MatRef == 1:
				mat.IOR = float(GVal)
				MatRef = 0
		elif Line.find("map") != -1:
			image = ''
			temp = Line.split()
			GVal = int(temp[1])
			if GVal != -1:
				if MatDiff == 2:
					DifIMG = Blender.Texture.New('Dif.'+mat.getName())
					DifIMG.setType('Image')
					try:
						DifIMG.image = Blender.Image.Load(MapList[GVal])
					except:
						print MapList[GVal],'is Not Found'
					mat.setTexture(0,DifIMG,Blender.Texture.TexCo.UV,Blender.Texture.MapTo.COL)
					MatDiff = 0
				elif MatBum == 2:
					BumIMG = Blender.Texture.New('Bum.'+mat.getName())
					BumIMG.setType('Image')
					try:
						BumIMG.image = Blender.Image.Load(MapList[GVal])
					except:
						print MapList[GVal],'is Not Found'
					mat.setTexture(1,BumIMG,Blender.Texture.TexCo.UV,Blender.Texture.MapTo.NOR)
					MatBum = 0
				elif MatTrn == 2:
					TrnIMG = Blender.Texture.New('Trn.'+mat.getName())
					TrnIMG.setType('Image')
					try:
						TrnIMG.image = Blender.Image.Load(MapList[GVal])
					except:
						print MapList[GVal],'is Not Found'
					mat.setTexture(2,TrnIMG,Blender.Texture.TexCo.UV,Blender.Texture.MapTo.ALPHA)
					MatTrn = 0
				elif MatAlp == 2:
					AlpIMG = Blender.Texture.New('Alp.'+mat.getName())
					AlpIMG.setType('Image')
					try:
						AlpIMG.image = Blender.Image.Load(MapList[GVal])
					except:
						print MapList[GVal],'is Not Found'
					mat.setTexture(3,AlpIMG,Blender.Texture.TexCo.UV,Blender.Texture.MapTo.ALPHA)
					MatAlp = 0
				elif MatGlw == 2:
					GlwIMG = Blender.Texture.New('Glw.'+mat.getName())
					GlwIMG.setType('Image')
					try:
						GlwIMG.image = Blender.Image.Load(MapList[GVal])
					except:
						print MapList[GVal],'is Not Found'
					mat.setTexture(4,GlwIMG,Blender.Texture.TexCo.UV,Blender.Texture.MapTo.EMIT)
					MatGlw = 0
				elif MatRefC == 2:
					RefCIMG = Blender.Texture.New('RefC.'+mat.getName())
					RefCIMG.setType('Image')
					try:
						RefCIMG.image = Blender.Image.Load(MapList[GVal])
					except:
						print MapList[GVal],'is Not Found'
					mat.setTexture(5,DifIMG,Blender.Texture.TexCo.UV,Blender.Texture.MapTo.REF)
					MatRefC = 0
				elif MatRef == 2:
					RefIMG = Blender.Texture.New('Ref.'+mat.getName())
					RefIMG.setType('Image')
					try:
						RefIMG.image = Blender.Image.Load(MapList[GVal])
					except:
						print MapList[GVal],'is Not Found'
					mat.setTexture(6,DifIMG,Blender.Texture.TexCo.UV,Blender.Texture.MapTo.REF)
					MatRef = 0
		Line = file.readline()

#***********************************************************
# ObjectGet
#***********************************************************
def ObjectGet(file,MatCnt):
	print "Object",
	objName = 'nill'
	vertList = []
	GroupName = []
	GroupList = []
	uvList = []
	faceList = []
	facev = []
	matnList = []
	mtList =[]
	VertNum = 0
	for Blmat in Blender.Material.Get():
		matnList.append(Blmat.getName())
	Line = file.readline()
	while Line[0] != "}":
		if len(Line) == 0: pass
		elif Line.find("Name") != -1:
			temp = Line.split()
			print temp[1]
			objName = temp[1]
			ob = NMesh.New(objName)
		elif Line.find(" Mesh ") != -1:
			temp = Line.split()
			tmp = temp[1].strip("\"")+','+temp[2]
			GroupName.append(str(tmp))
		elif Line.find("Vertexs") != -1:
			Line = file.readline()
			while Line.find("}") == -1:
				if len(Line) == 0: pass
				temp = Line.split()
				tmp = temp[0]+','+str(VertNum)
				GroupList.append(str(tmp))
				Verts = temp[1].split(',')
				vertList.append(NMesh.Vert(float(Verts[1]),float(Verts[0])*(-1),float(Verts[2])))
				VertNum += 1
				Line = file.readline()
		elif Line.find("UVs") != -1:
			Line = file.readline()
			while Line.find("}") == -1:
				if len(Line) == 0: pass
				temp = Line.split()
				Uvs = temp[1].split(',')
				uvList.append((float(Uvs[1]),float(Uvs[2])))
				Line = file.readline()
		elif Line.find("Polygons") != -1:
			Line = file.readline()
			nm = Blender.NMesh.GetRaw()
			while Line.find("}") == -1:
				if len(Line) == 0: pass
				fac = NMesh.Face()
				temp = Line.split()
				#if temp[6] == "-1": mat = 1
				#else: mat = temp[6]
				mat = temp[6]
				matFlg = 0
				id = 0
				for memat in mtList:
					if memat.getName() == matnList[int(mat)+MatCnt]:
						matFlg = 1
						break
					else: id += 1
				if matFlg == 0:
					mtList.append(Blender.Material.Get(matnList[int(mat)+MatCnt]))
				fac.materialIndex = id
				facen = temp[1].split(',')
				if len(uvList) != 0:uvn = temp[5].split(',')
				vn = len(facen)
				id = vn -1
				for i in range(vn):
					fac.v.append(vertList[int(facen[id])])
					if len(uvList) != 0:
						fac.uv.append(uvList[int(uvn[id])])
					id -= 1
				faceList.append(fac)
				Line = file.readline()
			nm.verts = vertList
			nm.faces = faceList
			nm.materials = mtList
			NewMesh = Blender.NMesh.PutRaw(nm,objName)
			mesh = NewMesh.getData()
			for GNum in GroupName:
				name = GNum.split(",")
				print "Group "+name[0]
				mesh.addVertGroup(name[0])
				GList = []
				for GLT in GroupList:
					tmp = GLT.split(",")
					if tmp[0] == name[1]:
						GList.append(int(tmp[1]))
				mesh.assignVertsToGroup(name[0],GList,1.0,'add')
			#NewMesh.name = objName
		Line = file.readline()

#***********************************************************
# File Selection
#***********************************************************
def File_Selection(filename):
	if filename.find('.mcl') != -1:
		Import(filename)
	else: print filename," is Not MarbleCLAY File"

Blender.Window.FileSelector(File_Selection,"Open mcl")
