#!BPY

"""
Name :'Redqueen Beta...(rrt)'
Blender: 234
Group: 'Export'
Tip: 'Export to Redqueen File Format'
"""

#************************************************************
# Blender to Redqueen Rendering Tool 2.0 Beta Exporter
# Original RRT_Exporter (2004/12/05) by Nao
#************************************************************

#************************************************************
# Import Module
#************************************************************
import Blender
from Blender import Types, Object, NMesh, Texture, Image,  Material, Draw
from math import *
from string import *

#************************************************************
# Main
#************************************************************

def export(filename):
	print '\n'+filename+" Exporting"
	file = open(filename, "w")
	renderSettingOut(file)
	currentCameraOut(file)
	backgroundOut(file)
	lightOut(file)
	imageOut(file)
	materialsOut(file)
	meshOut(file)
	file.close()
	print "Finish!"
	print filename+" OK"

def renderSettingOut(file):
	print "Render",
	file.write("renderer\n")
	file.write("\ttracedepth 7\n")
	file.write("\tcutoffcolor 0.0 0.0 0.0\n")
	file.write("\tambient 0 0 0\n")
	file.write("\tgisample 2\n")
	file.write("\tgimultiple 0.85\n")
	file.write("\tgiclip 5 5 5\n")
	file.write("\tfinalgathering 10\n")
	file.write("\tphotonsearchradius 15\n")
	file.write("end\n")
	print "OK"

def currentCameraOut(file):
	print "Camera",
	camera = Blender.Scene.getCurrent().getCurrentCamera()
	l = camera.getLocation()
	if camera.getParent() != None:
		p = camera.getParent()
		l = [0,0,0]
		l = p.getLocation()
		e = [0,0,0]
		e[0] = p.RotX
		e[1] = p.RotY
		e[2] = p.RotZ
		camera.setEuler(e[0],e[1],e[2])
	scene = Blender.Scene.getCurrent().getRenderingContext()
	file.write( "camera\n\tpinhole\n\toversampling 7\n\twidth "+str(scene.imageSizeX())+"\n\theight "+str(scene.imageSizeY())+"\n\tverticalfov "+str(round(360 * atan((scene.imageSizeX() / scene.imageSizeY()) * 12.5 / (camera.getData().lens + 2.75)) / pi )) + "\n" )
	#Camera Settings:  Copy & Paste From Eeshlo code .... Thanks!
	camtx = camera.matrix
	loc, size = tuple(l), tuple(camera.size)
	isy, isz = size[1], size[2]
	if isy!=0.0: isy = 1.0/isy
	if isz!=0.0: isz = 1.0/isz
	lookdir = (-camtx[2][0]*isz, -camtx[2][1]*isz, -camtx[2][2]*isz)
	lookAt = (loc[0] + lookdir[0], loc[1] + lookdir[1], loc[2] + lookdir[2])
	up = (camtx[1][0]*isy, camtx[1][1]*isy, camtx[1][2]*isy)

	file.write( '\tlocation %f %f %f\n' % loc )
	file.write( '\tlookat %f %f %f\n' % lookAt )
	#file.write( '\tupvector %f %f %f\n' % (loc[0]+up[0], loc[1]+up[1], loc[2]+up[2]) )
	file.write( '\tupvector %f %f %f\n' % up )

	file.write("end\n")
	print 'OK'

def backgroundOut(file):
	print "Sky",
	world = Blender.World.Get("World")
	file.write("background\n")
	ColorWrite(file,world.getHor()[0],world.getHor()[1],world.getHor()[2],'color')
	file.write( "end\n" )
	print "OK"

def lightOut(file):
	print "Light",
	Lmask = GetActiveLayer()
	for ob in Blender.Object.Get():
		if (Lmask & ob.Layer) == ob.Layer:
			if ob.getType() =='Lamp':
				lamp = ob.getData()
				lType = lamp.getType()
				if lType in ( 0, 2): #Lamp Make PointLight
					pos = ob.getLocation()
					col = lamp.getCol()
					El = lamp.getEnergy()
					dist = lamp.getDist()
					if lamp.getMode() & lamp.Modes['Sphere']:
						file.write("spherelight\n")
						file.write('\tradius %s\n' % str(dist/2))
						file.write('\tlocation %f %f %f\n' % pos)
						ColorWrite(file,col[0]*El,col[1]*El,col[2]*El,'color')
						file.write( "end\n" )
					else:
						file.write("pointlight\n")
						file.write('  location %f %f %f\n' % pos)
						ColorWrite(file,col[0]*El,col[1]*El,col[2]*El,'color')
						file.write("end\n")
				elif lType in ( 1, 3): #SunLight & HmiLight Make Parallellight
					size = tuple(ob.size)
					isz = size[2]
					if isz!=0.0: isz = 1.0/isz
					LiDirection = ob.getMatrix()
					DX = str(round(LiDirection[2][0]*isz,6))
					DY = str(round(LiDirection[2][1]*isz,6))
					DZ = str(round(LiDirection[2][2]*isz,6))
					file.write("parallellight\n")
					file.write("\tdirection "+DX+" "+DY+" "+DZ+"\n")
					col = lamp.getCol()
					ColorWrite(file,col[0],col[1],col[2],'color')
					file.write("end\n")
	print "OK"

def imageOut(file):
	print "Image",
	for mat in Blender.Material.Get():
		mtextures = mat.getTextures()
		for mtex in mtextures:
			if mtex is not None and mtex.tex.type == Texture.Types.IMAGE:
				imageName = replace(mtex.tex.image.filename,"\\","/")
				file.write("image\n")
				file.write('\tname %s\n' % imageName)
				file.write("end\n")
	print "Ok"

def materialsOut(file):
	print "Material"
	for mat in Blender.Material.Get():
		print mat.getName(),
		file.write("material\n\tname "+mat.getName())
		if mat.IOR <> 1:
			file.write("\n\trefractionindex "+str(round(mat.IOR,6))+"\n")
		else:
			file.write("\n\trefractionindex "+str(round(mat.getHardness(),6))+"\n")
		mtextures = mat.getTextures()
		RGB = mat.getRGBCol()
		ColorWrite(file,RGB[0],RGB[1],RGB[2],'color')
		for mtex in mtextures:
			if mtex is not None and mtex.tex.type == Texture.Types.IMAGE and mtex.mapto == Texture.MapTo.COL:
				difImage = replace(mtex.tex.image.filename,"\\","/")
				file.write("\tcolormap "+difImage+"\n")
		#Ref = mat.getRef()
		Ref = 0
		if Ref > 0:
			Mir = mat.getMirCol()
			ColorWrite(file,Ref*Mir[0],Ref*Mir[1],Ref*Mir[2],'reflection')
		Spc = mat.getSpec()
		if Spc > 0:
			SpcCol = mat.getSpecCol()
			ColorWrite(file,SpcCol[0]*Spc,SpcCol[1]*Spc,SpcCol[2]*Spc,'specular')
			ColorWrite(file,SpcCol[0]*Spc,SpcCol[1]*Spc,SpcCol[2]*Spc,'glossiness')
		ALp = mat.getAlpha()
		if ALp < 1:
			ColorWrite(file,1-ALp,1-ALp,1-ALp,'transparency')
		EMt = mat.getEmit()
		if EMt > 0:
			ColorWrite(file,RGB[0]*EMt,RGB[1]*EMt,RGB[2]*EMt,'emission')
		for mtex in mtextures:
			if mtex is not None and mtex.tex.type == Texture.Types.IMAGE and mtex.mapto == Texture.MapTo.NOR:
				file.write("\tbumpiness "+ str(mtex.tex.noiseDepth)+"\n")
				bumpImage = replace(mtex.tex.image.filename,"\\","/")
				file.write("\tbumpinessmap "+bumpImage+"\n")

		file.write("end\n")
		print "OK"

def meshOut(file):
	print "Mesh Out"
	Lmask = GetActiveLayer()
	Sum = 0
	for ob in Blender.Object.Get():
		if (Lmask & ob.Layer) == ob.Layer:
			if ob.getType() == 'Mesh':
				if ob.getName() == 'RQArea':
					RQAreaLight(ob, mat, file)
				else:
					Mesh = ob.getData()
					Mesh2 = Blender.NMesh.GetRawFromObject(ob.getName())
					uvFlag = 0
					if Mesh2.hasFaceUV():
						uvFlag = 1
					vertexhash = {}
					vertexList = []
					triangleList = []
					temp = ['','','','']
					id = 0
					for fac in Mesh2.faces:
						nv = len(fac.v)
						for i in range(nv):
							vertex = MatrixTransform((fac.v[i].co[0],fac.v[i].co[1],fac.v[i].co[2]),ob.matrix)
							if uvFlag:uv = [fac.uv[i][0],fac.uv[i][1]*-1]
							else: uv = 0,0
							hash = vertex[0] * 3 + vertex[1] * 5 + vertex[2] * 7 + uv[0] * 11 + uv[1] * 13
							temp[i] = str(hash)
							try: dummy = vertexhash[str(hash)]
							except:
								vertexhash[str(hash)] = id
								vertexList.append([vertex[0],vertex[1],vertex[2],uv[0],uv[1]])
								id += 1
						triangleList.append([fac.materialIndex,vertexhash[temp[0]], vertexhash[temp[1]], vertexhash[temp[2]]])
						if nv == 4:
							triangleList.append([fac.materialIndex,vertexhash[temp[2]], vertexhash[temp[3]], vertexhash[temp[0]]])
					result =''
					print ob.getName(),'vertices',
					if uvFlag == 1:
						for i in range(len(vertexList)):
							result = result + 'vertex position %f %f %f uv %f %f end\n' % (vertexList[i][0], vertexList[i][1], vertexList[i][2],vertexList[i][3],vertexList[i][4])
					else:
						for i in range(len(vertexList)):
							result = result + 'vertex position %f %f %f end\n' % (vertexList[i][0], vertexList[i][1], vertexList[i][2])
					file.write(result)
					print 'OK'
					for mat in Mesh.getMaterials(-1):
						print ob.getName(),'of',mat.name,
						file.write("object\n\tname "+ob.getName()+"_"+mat.name+"\n\tassignmaterial "+mat.name+"\n")
						file.write("\tsmoothing "+str(Mesh.getMaxSmoothAngle())+"\n")
						if uvFlag == 1: file.write("\tuvmap\n")
						result = ''
						for i in range(len(triangleList)):
							if Mesh2.materials[triangleList[i][0]].getName() == mat.name:
								result = result + '\ttriangle %i %i %i\n' % (triangleList[i][1]+Sum,triangleList[i][2]+Sum,triangleList[i][3]+Sum)
						file.write(result)
						file.write("end\n")
						print 'OK'
					Sum = Sum + len(vertexList)

#***********************************************************
# Area Light
#***********************************************************
def RQAreaLight(ob,mat,file):
	print ob.getName(),'Transrate to AreaLight'
	AreaLight = Blender.NMesh.GetRawFromObject(ob.getName())
	loc = []
	for fac in AreaLight.faces:
		try:
			mat = Blender.Material.Get(AreaLight.materials[fac.materialIndex].getName())
			RGB = mat.getRGBCol()
		except:
			RGB = 1,1,1
		for nv in range(len(fac.v)):
			loc.append(MatrixTransform(LSAdjust((fac.v[nv].co[0],fac.v[nv].co[1],fac.v[nv].co[2])),ob.matrix))
		file.write('arealight\n')
		file.write('\tcolor    %f %f %f\n' % (RGB[0],RGB[1],RGB[2]))
		file.write('\tlocation %f %f %f\n' % (loc[0][0],loc[0][1],loc[0][2]))
		file.write('\tvectoru  %f %f %f\n' % (loc[1][0] - loc[0][0],loc[1][1] - loc[0][1],loc[1][2] - loc[0][2]))
		file.write('\tvectorv  %f %f %f\n' % (loc[2][0] - loc[1][0],loc[2][1] - loc[1][1],loc[2][2] - loc[1][2]))
		file.write('end\n')
		if len(fac.v) == 4:
			file.write('arealight\n')
			file.write('\tcolor    %f %f %f\n' % (RGB[0],RGB[1],RGB[2]))
			file.write('\tlocation %f %f %f\n' % (loc[2][0],loc[2][1],loc[3][2]))
			file.write('\tvectoru  %f %f %f\n' % (loc[0][0] - loc[3][0],loc[0][1] - loc[3][1],loc[0][2] - loc[3][2]))
			file.write('\tvectorv  %f %f %f\n' % (loc[3][0] - loc[2][0],loc[3][1] - loc[2][1],loc[3][2] - loc[2][2]))
			file.write('end\n')

def LSAdjust(vert):
	x, y, z = vert
	x = x * 8
	y = y * 8
	z = z * 8
	vert = [x,y,z]
	return vert

#***********************************************************
# Edge Direction Get
#***********************************************************
def GetEdgiDir(vert1, vert2):
	retx = vert2[0] - vert1[0]
	rety = vert2[1] - vert1[1]
	retz = vert2[2] - vert2[2]

	return [retx,rety,retz]

#***********************************************************
# Color Write Wrapper
#***********************************************************
def ColorWrite(file,R,G,B,head):
	r = str(round(R,3))
	g = str(round(G,3))
	b = str(round(B,3))
	file.write("\t"+head+" "+r+" "+g+" "+b+"\n")

#***********************************************************
# Layer Utility
#***********************************************************
def GetActiveLayer():
	maskColection = [0,1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768,65536,131072,262144,524288,1048573]
	Layermask = 0
	layers = Blender.Window.ViewLayer()
	for i in range(len(layers)):
		Layermask = Layermask + maskColection[layers[i]]
	return Layermask

#***********************************************************
# Matrix Utility
#***********************************************************
def MatrixTransform(vert,matrix):
	x, y, z = vert
	retx = x*matrix[0][0] + y*matrix[1][0] + z*matrix[2][0] + matrix[3][0]
	rety = x*matrix[0][1] + y*matrix[1][1] + z*matrix[2][1] + matrix[3][1]
	retz = x*matrix[0][2] + y*matrix[1][2] + z*matrix[2][2] + matrix[3][2]
	vert = [retx,rety,retz]
	return vert

#***********************************************************
# File Selection
#***********************************************************
def fs_callback(filename):
	if filename.find('.rrt', -4) <= 0: filename += '.rrt'
	export(filename)
Blender.Window.FileSelector(fs_callback,"Save rrt")
