#!BPY

"""
Name :'Redqueen...(rrt)'
Blender: 234
Group: 'Export'
Tip: 'Export to Redqueen File Format'
"""

#************************************************************
# Blender to Redqueen Rendering Tool 2.0 Free Exporter
# Original RRT_Exporter (2004/12/15) by Nao
#************************************************************

#************************************************************
# Import Module
#************************************************************
import Blender
from Blender import Types, Object, NMesh, Texture, Image,  Material, Draw
from math import *
from string import *

#************************************************************
# Main
#************************************************************

def export(filename):
	print''
	print filename+" Exporting"
	file = open(filename, "w")
	startScene(file)
	renderSettingOut(file)
	currentCameraOut(file)
	backgroundOut(file)
	lightOut(file)
	materialsOut(file)
	meshOut(file)
	endScene(file)
	file.close()
	print "Finish!"
	print filename+" OK"

#************************************************************
# Head & foot
#************************************************************
def startScene(file):
	file.write('<?xml version="1.0" encoding="Shift_JIS"?>')
	file.write("\n<redqueen>\n")

def endScene(file):
	file.write("</redqueen>\n")

#************************************************************
# Renderer Setting
#************************************************************
def renderSettingOut(file):
	finalgatherray = 128	#default 11
	tracedepth = 16	#default 7
	diffusedepth = 4	#default 2
	diffusealbedo = 0.85	#default 0.85
	causticsphotongatherradius = 1.0	#default 1.0
	cutoff = 0, 0, 0	#default 0, 0, 0
	ambient = 0.6, 0.6, 0.6	#default 0, 0, 0
	print "Render",
	file.write('\t<renderer\n')
	file.write('\t\tdiffusealbedo="%f"\n' % diffusealbedo)
	file.write('\t\tdiffusedepth="%i"\n' % diffusedepth)
	file.write('\t\tfinalgatherray="%i"\n' % finalgatherray)
	file.write('\t\tcausticsphotongatherradius="%f"\n' % causticsphotongatherradius)
	file.write('\t\ttracedepth="%i">\n' % tracedepth)
	file.write('\t\t<cutoff r="%f" g="%f" b="%f" />\n' % (cutoff[0],cutoff[1],cutoff[2]))
	file.write('\t\t<ambient r="%f" g="%f" b="%f" />\n'% (ambient[0],ambient[1],ambient[2]))
	file.write('\t</renderer>\n')
	print "OK"

#************************************************************
# Camera Setting
#************************************************************
def currentCameraOut(file):
	print "Camera",
	camera = Blender.Scene.getCurrent().getCurrentCamera()
	l = camera.getLocation()
	if camera.getParent() != None:
		p = camera.getParent()
		l = [0,0,0]
		l = p.getLocation()
		e = [0,0,0]
		e[0] = p.RotX
		e[1] = p.RotY
		e[2] = p.RotZ
	scene = Blender.Scene.getCurrent().getRenderingContext()
	CamType = "pinhole" #ex "panorama"
	Oversampling = 12 #default 7
	Blur = 0.0 #default 0.0
	WidthPix = scene.imageSizeX()
	HeightPix = scene.imageSizeY()
	FealdOfView = round(360 * atan((scene.imageSizeX() / scene.imageSizeY()) * 12.5 / (camera.getData().lens + 2.75)) / pi )
	file.write('\t<camera type="%s" oversampling="%i" width="%i" height="%i" fov="%f" blur="%f">\n' % (CamType,Oversampling,WidthPix,HeightPix,FealdOfView,Blur))
	#file.write( "\t<camera type=\"pinhole\" oversampling=\"7\"" + " width=\""+str(scene.imageSizeX())+"\" height=\""+str(scene.imageSizeY())+"\" fov=\""+str(round(360 * atan((scene.imageSizeX() / scene.imageSizeY()) * 12.5 / (camera.getData().lens + 2.75)) / pi )) + "\">\n" )
	#Camera Settings:  Copy & Paste From Eeshlo code .... Thanks!
	camtx = camera.matrix
	loc, size = tuple(l), tuple(camera.size)
	isy, isz = size[1], size[2]
	if isy!=0.0: isy = 1.0/isy
	if isz!=0.0: isz = 1.0/isz
	lookdir = (-camtx[2][0]*isz, -camtx[2][1]*isz, -camtx[2][2]*isz)
	lookAt = (loc[0] + lookdir[0], loc[1] + lookdir[1], loc[2] + lookdir[2])
	up = (camtx[1][0]*isy, camtx[1][1]*isy, camtx[1][2]*isy)

	file.write( '\t\t<location x="%f" y="%f" z="%f" />\n' % loc )
	file.write( '\t\t<target x="%f" y="%f" z="%f" />\n' % lookAt )
	file.write( '\t\t<upvector x="%f" y="%f" z="%f" />\n' % up )
	#file.write( '\t\t<upvector x="%f" y="%f" z="%f" />\n' % (loc[0]+up[0], loc[1]+up[1], loc[2]+up[2]) )

	file.write('\t</camera>\n')
	print "OK"

#************************************************************
# Background Setting
#************************************************************
def backgroundOut(file):
	print "Sky",
	SkyType = "colored"
	world = Blender.World.Get("World")
	file.write('\t<sky type="%s">\n' % SkyType)
	ColorWrite(file,world.getHor()[0],world.getHor()[1],world.getHor()[2],2)
	file.write( '\t</sky>\n' )
	print "OK"

#************************************************************
# Light Setting
#************************************************************
def lightOut(file):
	print "Light",
	Lmask = GetActiveLayer()
	for ob in Blender.Object.Get():
		if (Lmask & ob.Layer) == ob.Layer:
			if ob.getType() == 'Lamp':
				lamp = ob.getData()
				lType = lamp.getType()
				if lType in ( 0, 2): #Lamp Make PointLight
					pos = ob.getLocation()
					col = lamp.getCol()
					El = lamp.getEnergy()
					dist = lamp.getDist()
					if lamp.getMode() & lamp.Modes['Sphere']:
						file.write('\t<spherelight radius="%f">\n' % round(dist/2,6))
						file.write('\t\t<location x="%f" y="%f" z="%f"/>\n' % (round(pos[0],6),round(pos[1],6),round(pos[2],6)))
						#ColorWrite(file,col[0]*El,col[1]*El,col[2]*El,2)
						ColorWrite(file,col[0]*(El*2)*(dist*2),col[1]*(El*2)*(dist*2),col[2]*(El*2)*(dist*2),2)
						file.write( '\t</spherelight>\n' )
					else:
						file.write('\t<pointlight>\n')
						file.write('\t\t<location x="%f" y="%f" z="%f"/>\n' % (round(pos[0],6),round(pos[1],6),round(pos[2],6)))
						ColorWrite(file,col[0]*(El*2)*(dist*2),col[1]*(El*2)*(dist*2),col[2]*(El*2)*(dist*2),2)
						file.write('\t</pointlight>\n')
				elif lType in ( 1, 3): #SunLight & HmiLight Make Parallellight
					size = tuple(ob.size)
					isz = size[2]
					if isz!=0.0: isz = 1.0/isz
					LiDirection = ob.getMatrix()
					DX = round(LiDirection[2][0]*isz,6)
					DY = round(LiDirection[2][1]*isz,6)
					DZ = round(LiDirection[2][2]*isz,6)
					file.write('\t<parallellight>\n')
					file.write('\t\t<direction x="%f" y="%f" z="%f\"/>\n' % (DX,DY,DZ))
					col = lamp.getCol()
					ColorWrite(file,col[0],col[1],col[2],2)
					file.write('\t</parallellight>\n')
	print "OK"

#************************************************************
# Texture Map Setting
#************************************************************
def TexMapOut( TexType, mtextures, file ):
	for mtex in mtextures:
		if mtex is not None and mtex.tex.type == Texture.Types.IMAGE and mtex.mapto == TexType:
			ImageName = replace(mtex.tex.image.filename,"\\","/")
			file.write('\t\t\t<image name="%s"/>\n' % ImageName)

#************************************************************
# Material Setting
#************************************************************
def materialsOut(file):
	print "Material"
	for mat in Blender.Material.Get():
		print mat.getName(),
		file.write('\t<material name="%s" refractionindex="' % mat.getName())
		if mat.getIOR() <> 1 :
			file.write(str(round(mat.getIOR(),6))+'">\n')
		else:
			file.write(str(round(mat.getHardness(),6))+'">\n')
		mtextures = mat.getTextures()
		RGB = mat.getRGBCol()
		file.write('\t\t<diffusion>\n')
		ColorWrite(file,RGB[0],RGB[1],RGB[2],3)
		TexMapOut( Texture.MapTo.COL, mtextures, file )
		file.write('\t\t</diffusion>\n')
		Ref = mat.getRayMirr()
		#Ref = 0
		if Ref > 0:
			Mir = mat.getMirCol()
			file.write('\t\t<reflection>\n')
			ColorWrite(file,Ref*Mir[0],Ref*Mir[1],Ref*Mir[2],3)
			file.write('\t\t</reflection>\n')
		Spc = mat.getSpec()
		if Spc > 0:
			SpcCol = mat.getSpecCol()
			file.write('\t\t<specularity>\n')
			ColorWrite(file,SpcCol[0]*Spc,SpcCol[1]*Spc,SpcCol[2]*Spc,3)
			file.write('\t\t</specularity>\n')
			file.write('\t\t<glossiness>\n')
			ColorWrite(file,SpcCol[0]*Spc,SpcCol[1]*Spc,SpcCol[2]*Spc,3)
			file.write('\t\t</glossiness>\n')
		ALp = mat.getAlpha()
		ALpFlg = 0
		if ALp < 1:
			file.write('\t\t<transparency>\n')
			ColorWrite(file,1-ALp,1-ALp,1-ALp,3)
			TexMapOut( Texture.MapTo.ALPHA, mtextures, file )
			file.write('\t\t</transparency>\n')
			ALpFlg = 1
		if ALpFlg == 0:
			ALpImage = ''
			for mtex in mtextures:
				if mtex is not None and mtex.tex.type == Texture.Types.IMAGE and mtex.mapto == Texture.MapTo.ALPHA:
					ALpImage = replace(mtex.tex.image.filename,"\\","/")
			if ALpImage <> '':
				file.write('\t\t<trimming>\n')
				file.write('\t\t\t<image name="%s"/>\n' % ALpImage)
				file.write('\t\t</trimming>\n')
		EMt = mat.getEmit()
		if EMt > 0:
			file.write('\t\t<emission>\n')
			ColorWrite(file,RGB[0] * EMt,RGB[1] * EMt,RGB[2] * EMt,3)
			TexMapOut( Texture.MapTo.EMIT, mtextures, file )
			file.write('\t\t</emission>\n')
		for mtex in mtextures:
			if mtex is not None and mtex.tex.type == Texture.Types.IMAGE and mtex.mapto == Texture.MapTo.NOR:
				file.write('\t\t<bumpiness strength="%f">\n' % mtex.tex.noiseDepth)
				bumpImage = replace(mtex.tex.image.filename,"\\","/")
				file.write('\t\t\t<image name="%s"/>\n' % bumpImage)
				file.write('\t\t</bumpiness>\n')

		file.write('\t</material>\n')
		print "OK"

#************************************************************
# Mesh Object Setting
#************************************************************
def meshOut(file):
	print "Mesh Out"
	Lmask = GetActiveLayer()
	for ob in Blender.Object.Get():
		if (Lmask & ob.Layer) == ob.Layer:
			if ob.getType() == 'Mesh':
				if ob.getName() == 'RQArea':
					print ob.getName(),'Trancerate to AreaLight'
					AreaLight = Blender.NMesh.GetRawFromObject(ob.getName())
					loc = []
					for fac in AreaLight.faces:
						try:
							mat = Blender.Material.Get(AreaLight.materials[fac.materialIndex].getName())
							RGB = mat.getRGBCol() #LCAdjust(mat.getRGBCol())
						except:
							RGB = 1,1,1
						for nv in range(len(fac.v)):
							loc.append(MatrixTransform(LSAdjust((fac.v[nv].co[0],fac.v[nv].co[1],fac.v[nv].co[2])), ob.matrix))
						file.write('\t<arealight shadowray="190">\n')
						file.write('\t\t<location x="%f" y="%f" z="%f"/>\n' % (loc[0][0],loc[0][1],loc[0][2]))
						file.write('\t\t<vectoru x="%f" y="%f" z="%f"/>\n' % (loc[1][0] - loc[0][0],loc[1][1] - loc[0][1],loc[1][2] - loc[0][2]))
						file.write('\t\t<vectorv x="%f" y="%f" z="%f"/>\n' % (loc[2][0] -loc[1][0],loc[2][1] - loc[1][1],loc[2][2] - loc[1][2]))
						ColorWrite(file,RGB[0],RGB[1],RGB[2],3)
						#file.write('\t\t<color r="80" g="80" b="80"/>\n')
						if len(fac.v) == 4:
							file.write('\t</arealight>\n')
							file.write('\t<arealight shadowray="190">\n')
							file.write('\t\t<location x="%f" y="%f" z="%f"/>\n' % (loc[2][0],loc[2][1],loc[2][2]))
							file.write('\t\t<vectoru x="%f" y="%f" z="%f"/>\n' % (loc[0][0] - loc[3][0],loc[0][1] - loc[3][1],loc[0][2] - loc[3][2]))
							file.write('\t\t<vectorv x="%f" y="%f" z="%f"/>\n' % (loc[3][0] -loc[2][0],loc[3][1] - loc[2][1],loc[3][2] - loc[2][2]))
							ColorWrite(file,RGB[0],RGB[1],RGB[2],3)
							#file.write('\t\t<color r="80" g="80" b="80"/>\n')
						file.write('\t</arealight>\n')
				else:
					Mesh = ob.getData()
					file.write('\t<object name="%s" smoothing="%i">\n' % (ob.getName(),Mesh.getMaxSmoothAngle()))
					mat = []
					uvFlag = 0
					for mat in Mesh.getMaterials(-1):
						print ob.getName(),"of",mat.name,"vertices",
						file.write('\t\t<part name="%s" material="%s">\n' % (ob.getName()+"_"+mat.name,mat.name))
						file.write('\t\t\t<vertices>\n')
						Mesh2 = Blender.NMesh.GetRawFromObject(ob.getName())
						if Mesh2.hasFaceUV():
							uvFlag = 1
						vertexhash = {}
						vertexList = []
						triangleList = []
						id = 0
						temp = ['', '', '', '']
						for fac in Mesh2.faces:
							mIDX = fac.materialIndex
							if (mIDX > len(Mesh2.materials)) or (mIDX < 0): mIDX = 1
							if mat.name == Mesh2.materials[mIDX].getName():
								nv = len(fac.v)
								result = ''
								for i in range(nv):
									vertex = MatrixTransform((fac.v[i].co[0], fac.v[i].co[1], fac.v[i].co[2]), ob.matrix)
									if uvFlag: uv = [fac.uv[i][0],fac.uv[i][1]*-1]
									else: uv = 0, 0
									hash = vertex[0] * 3 + vertex[1] * 5 + vertex[2] * 7 + uv[0] * 11 + uv[1] * 13
									temp[i] = str(hash)
									try:
										dummy = vertexhash[str(hash)]
									except:
										vertexhash[str(hash)] = id
										if uvFlag == 1:
											result = result + '\t\t\t\t<vertex x="%f" y="%f" z="%f" u="%f" v="%f"/>\n' % (vertex[0], vertex[1], vertex[2],uv[0],uv[1])
										else:
											result = result + '\t\t\t\t<vertex x="%f" y="%f" z="%f"/>\n' % (vertex[0], vertex[1], vertex[2])
										id += 1
								file.write(result)
								triangleList.append([vertexhash[temp[0]], vertexhash[temp[1]], vertexhash[temp[2]]])
								if nv == 4:
									triangleList.append([vertexhash[temp[2]], vertexhash[temp[3]], vertexhash[temp[0]]])
						file.write('\t\t\t</vertices>\n')
						file.write('\t\t\t<triangles>\n')
						print "triangles",
						result = ''
						for i in range(len(triangleList)):
							result = '\t\t\t\t<triangle id0="%i" id1="%i" id2="%i"/>\n' % (triangleList[i][0], triangleList[i][1], triangleList[i][2])
							file.write(result)
						file.write('\t\t\t</triangles>\n')
						file.write('\t\t</part>\n')
						print "Ok"
					file.write('\t</object>\n')

#***********************************************************
# Color Write
#***********************************************************
def ColorWrite(file,R,G,B,Tab):
	temp = ""
	for i in range(Tab):
		temp = temp + "\t"
	r = round(R,6)
	g = round(G,6)
	b = round(B,6)
	file.write(temp+'<color r="%f" g="%f" b="%f"/>\n' % (r,g,b))

#***********************************************************
# Layer Option
#***********************************************************
def GetActiveLayer():
	maskColection = [0,1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768,65536,131072,262144,524288,1048573]
	Layermask = 0
	layers = Blender.Window.ViewLayer()
	for i in range(len(layers)):
		Layermask = Layermask + maskColection[layers[i]]
	return Layermask

#***********************************************************
# Light Adjust
#***********************************************************
def LSAdjust(vert):
	x, y, z = vert
	x = x * 8
	y = y * 8
	z = z * 8
	vert = [x,y,z]
	return vert

def LCAdjust(RGB):
	r, g, b = RGB
	r = r * 10
	g = g * 10
	b = b * 10
	RGB = [r,g,b]
	return RGB

#***********************************************************
# Matrix Utility
#***********************************************************
def MatrixTransform(vert,matrix):
	x, y, z = vert
	retx = x*matrix[0][0] + y*matrix[1][0] + z*matrix[2][0] + matrix[3][0]
	rety = x*matrix[0][1] + y*matrix[1][1] + z*matrix[2][1] + matrix[3][1]
	retz = x*matrix[0][2] + y*matrix[1][2] + z*matrix[2][2] + matrix[3][2]
	vert = [retx, rety, retz]
	return vert

#************************************************************
# File Selection
#************************************************************
def fs_callback(filename):
	if filename.find('.rrt', -4) <= 0: filename += '.rrt'
	export(filename)
Blender.Window.FileSelector(fs_callback,"Save rrt")

